/*
 *  ISEM - Instructional Sparc EMulator and tkisem
 *  Copyright (C) 1993, 1994, 1995, 1996
 *	Department of Computer Science,
 *      The University of New Mexico
 *
 *  Please send questions, comments, and bug reports to: isem@cs.unm.edu
 *
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

//-----------------------------------------------------------------------------
// square root instruction - see page 145 of SPARC Architecture Manual,
//				              Version 8
//-----------------------------------------------------------------------------

#if __GNUC__
#define UNUSED __attribute__ ((unused)) 
#else
#define UNUSED
#endif

static char rcsid[] UNUSED = "$Id: sqrt.cpp 1.1 Fri, 25 Oct 1996 18:04:04 -0600 maccabe $";

#include <math.h>

#include "Assert.h"
#include "sizedefs.h"
#include "Instruct.h"
#include "IU.h"
#include "FPU.h"

// fp_exception NV,NX,invalid_fp_register
// rounding performed -- sqrt rounds according to ANSI... however
//    do I have a rounding problem with the float since I
//    convert result from double.

int FloatingPointUnit::square_root(const Instruction& inst) { 
    switch (inst.opf()){
    case FSQRTs:
	_freg[inst.rd()] = (Float_s)sqrt( (Float_d)_freg[inst.rs2()] );
	break;
    case FSQRTd:
        if ((inst.rd() & 1) || (inst.rs2() & 1)) {
	    _ftt = invalid_fp_register;
	    return IntegerUnit::fp_exception;
        }
	_dreg[inst.rd()] = sqrt(_dreg[inst.rs2()]);
	break;
    case FSQRTq:
	_ftt = unimplemented_FPop;
	return unimplemented();
    default:
	Assert(0, "unknown opf: PLEASE EMAIL isem@cs.unm.edu with this error!");
    }

    _ftt = No_FPU_Trap;
    return 0;
}
